from __future__ import absolute_import

from nose.plugins import Plugin as NosePlugin
from nose.loader import TestLoader


class SybilLoader(TestLoader):

    def __init__(self, test_suite_func, config):
        super(SybilLoader, self).__init__(config)
        self.test_suite_func = test_suite_func

    def loadTestsFromModule(self, module, path=None, discovered=False):
        suite_func = getattr(module, self.test_suite_func, None)
        if suite_func is not None:
            return suite_func()

        return super(SybilLoader, self).loadTestsFromModule(
            module, path, discovered
            )


class Plugin(NosePlugin):

    name = 'sybil'

    enabled = True

    def options(self, parser, env):
        parser.add_option(
            "--test-suite-func", action="store",
            dest="test_suite_func",
            default='load_tests',
            help="A function in modules that will return a TestSuite. "
                 "Defaults to 'load_tests'.")

    def configure(self, options, config):
        self.test_suite_func = options.test_suite_func

    def prepareTestLoader(self, loader):
        return SybilLoader(self.test_suite_func, loader.config)



class SybilFailure(AssertionError):

    def __init__(self, example, result):
        super(SybilFailure, self).__init__((
            'Example at {}, line {}, column {} did not evaluate as expected:\n'
            '{}'
        ).format(example.document.path, example.line, example.column, result))
        self.example = example
        self.result = result


class Example(object):
    """
    This represents a particular example from a documentation source file.
    It is assembled from the :class:`~sybil.document.Document` and 
    :class:`~sybil.Region` the example comes from and is passed to the region's 
    evaluator.
    """

    def __init__(self, document, line, column, region, namespace):
        #: The :class:`~sybil.document.Document` from which this example came.
        self.document = document
        #: The absolute path of the :class:`~sybil.document.Document`.
        self.path = document.path
        #: The line number at which this example occurs in the
        #: :class:`~sybil.document.Document`.
        self.line = line
        #: The column number at which this example occurs in the
        #: :class:`~sybil.document.Document`.
        self.column = column
        #: The :class:`~sybil.Region` from which this example came.
        self.region = region
        #: The character position at which this example starts in the
        #: :class:`~sybil.document.Document`.
        self.start = region.start
        #: The character position at which this example ends in the
        #: :class:`~sybil.document.Document`.
        self.end = region.end
        #: The version of this example provided by the parser that yielded
        #: the :class:`~sybil.Region` containing it.
        self.parsed = region.parsed
        #: The :attr:`~sybil.document.Document.namespace` of the document from
        #: which this example came.
        self.namespace = namespace

    def __repr__(self):
        return '<Example path={} line={} column={} using {!r}>'.format(
            self.document.path, self.line, self.column, self.region.evaluator
        )

    def evaluate(self):
        evaluator = self.document.evaluator or self.region.evaluator
        result = evaluator(self)
        if result:
            raise SybilFailure(self, result)
